---
title: Version 2.4 release notes
author: Loïc Poullain
author_title: Fullstack developer and creator of FoalTS
author_url: https://github.com/LoicPoullain
author_image_url: https://avatars1.githubusercontent.com/u/13604533?v=4
image: blog/twitter-banners/version-2.4-release-notes.png
tags: [release]
---

![Banner](./assets/version-2.4-is-here/banner.png)

Version 2.4 of Foal has been released! Here are the improvements that it brings.

<!--truncate-->

## `$data` references for validation

Version 2.4 allows you to enable the AJV `$data` option so that you can use the verified data values as validators for other values.

*config/default.json*
```json
{
  "settings": {
    "ajv": {
      "$data": true
    }
  }
}
```

*Example of auth controller*
```typescript
import { Context, Post, ValidateBody } from '@foal/core';

export class AuthController {
  @Post('/signup')
  @ValidateBody({
    type: 'object',
    properties: {
      username: { type: 'string' },
      password: { type: 'string' },
      // "password" and "confirmPassword" should be identical.
      confirmPassword: {
        const: {
          $data: '1/password',
        },
        type: 'string',
      },
    }
    required: [ 'username', 'password', 'confirmPassword' ],
    additionalProperties: false
  })
  signup(ctx: Context) {
    // Do something.
  }
}

```

## Cache option for file downloading

Starting from version 2.4 the `Disk.createHttpResponse` method accepts an optional parameter to specify the value of the `Cache-Control` header.

```typescript
import { Context, dependency, Get } from '@foal/core';
import { Disk } from '@foal/storage';

import { User } from '../entities';

export class ProfileController {
  @dependency
  disk: Disk;

  @Get('/avatar')
  async readProfileImage(ctx: Context<User>) {
    return this.disk.createHttpResponse(ctx.user.avatar, {
      cache: 'no-cache'
    });
  }
```

## Bug fixes

See issue [#930](https://github.com/FoalTS/foal/issues/930).

## Contributors

[@ZakRabe](https://github.com/ZakRabe)